package App::bif::Version;
$App::bif::Version::VERSION = '0.1.0_11';
$App::bif::Version::COMMIT = 'fbb32fbad3dea98fdf729122ff985defd2c2149a';
$App::bif::Version::BRANCH = 'HEAD';
$App::bif::Version::DATE = '2014-04-18 11:05:14';
1;
